@@ -50,6 +50,7 @@ pub enum NetworkError {
 }
 
 impl NetworkError {
+    // FIXME (nkls): is unused and overlaps with `is_trivial`?
     pub fn is_fatal(&self) -> bool {
         match self {
             Self::Io(err) => [
@@ -66,7 +67,7 @@ impl NetworkError {
 
     pub fn is_trivial(&self) -> bool {
         match self {
-            NetworkError::Io(e) => {
+            Self::Io(e) => {
                 matches!(
                     e.kind(),
                     ErrorKind::BrokenPipe
